/*
    GNU linker script for STM32F401xE
*/

/* Specify the memory areas 256kB FLASH 128KB RAM */
MEMORY
{
    FLASH (rx)      : ORIGIN = 0x08000000, LENGTH = 0x040000 /* entire flash */
    FLASH_ISR (rx)  : ORIGIN = 0x08000000, LENGTH = 0x000200 /* 14,75k */
    FLASH_TEXT (rx) : ORIGIN = 0x08000200, LENGTH = 0x03FE00 /* sectors 5,6,7 are 128K */
    RAM (xrw)       : ORIGIN = 0x20000000, LENGTH = 0x010000
}

/* produce a link error if there is not this amount of RAM for these sections */
_minimum_stack_size = 2K;
_minimum_heap_size = 16K;

/* Define tho top end of the stack.  The stack is full descending so begins just
   above last byte of RAM.  Note that EABI requires the stack to be 8-byte
   aligned for a call. */
_estack = ORIGIN(RAM) + LENGTH(RAM);

/* define common sections and symbols */
INCLUDE common.ld

/* RAM extents for the garbage collector */
_ram_start = ORIGIN(RAM);
_ram_end = ORIGIN(RAM) + LENGTH(RAM);
_heap_start = _ebss; /* heap starts just after statically allocated memory */
_heap_end = 0x20014000; /* tunable */
